@@ -5,9 +5,8 @@ import os
 import re
 import socket
 import subprocess
-from subprocess import PIPE
-from subprocess import Popen
-from typing import List
+from subprocess import PIPE, Popen
+from typing import Iterable, List
 from warnings import warn
 
 LOCAL_IPS: List = []
@@ -16,26 +15,32 @@ PUBLIC_IPS: List = []
 LOCALHOST = ""
 
 
-def _uniq_stable(elems):
+def _uniq_stable(elems: Iterable) -> List:
     """uniq_stable(elems) -> list
 
     Return from an iterable, a list of all the unique elements in the input,
     maintaining the order in which they first appear.
     """
     seen = set()
-    return [x for x in elems if x not in seen and not seen.add(x)]
+    value = []
+    for x in elems:
+        if x not in seen:
+            value.append(x)
+            seen.add(x)
+    return value
 
 
 def _get_output(cmd):
     """Get output of a command, raising IOError if it fails"""
     startupinfo = None
     if os.name == "nt":
-        startupinfo = subprocess.STARTUPINFO()
-        startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
-    p = Popen(cmd, stdout=PIPE, stderr=PIPE, startupinfo=startupinfo)
+        startupinfo = subprocess.STARTUPINFO()  # type:ignore[attr-defined]
+        startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW  # type:ignore[attr-defined]
+    p = Popen(cmd, stdout=PIPE, stderr=PIPE, startupinfo=startupinfo)  # noqa
     stdout, stderr = p.communicate()
     if p.returncode:
-        raise IOError("Failed to run %s: %s" % (cmd, stderr.decode("utf8", "replace")))
+        msg = "Failed to run {}: {}".format(cmd, stderr.decode("utf8", "replace"))
+        raise OSError(msg)
     return stdout.decode("utf8", "replace")
 
 
@@ -64,7 +69,7 @@ def _requires_ips(f):
 
 
 # subprocess-parsing ip finders
-class NoIPAddresses(Exception):
+class NoIPAddresses(Exception):  # noqa
     pass
 
 
@@ -88,7 +93,7 @@ def _populate_from_list(addrs):
         LOCALHOST = "127.0.0.1"
         local_ips.insert(0, LOCALHOST)
 
-    local_ips.extend(["0.0.0.0", ""])
+    local_ips.extend(["0.0.0.0", ""])  # noqa
 
     LOCAL_IPS[:] = _uniq_stable(local_ips)
     PUBLIC_IPS[:] = _uniq_stable(public_ips)
@@ -146,7 +151,7 @@ def _load_ips_ipconfig():
 
 def _load_ips_netifaces():
     """load ip addresses with netifaces"""
-    import netifaces  # type: ignore
+    import netifaces  # type: ignore[import]
 
     global LOCALHOST
     local_ips = []
@@ -169,7 +174,7 @@ def _load_ips_netifaces():
         # we never found a loopback interface (can this ever happen?), assume common default
         LOCALHOST = "127.0.0.1"
         local_ips.insert(0, LOCALHOST)
-    local_ips.extend(["0.0.0.0", ""])
+    local_ips.extend(["0.0.0.0", ""])  # noqa
     LOCAL_IPS[:] = _uniq_stable(local_ips)
     PUBLIC_IPS[:] = _uniq_stable(public_ips)
 
@@ -182,7 +187,7 @@ def _load_ips_gethostbyname():
     global LOCALHOST
     try:
         LOCAL_IPS[:] = socket.gethostbyname_ex("localhost")[2]
-    except socket.error:
+    except OSError:
         # assume common default
         LOCAL_IPS[:] = ["127.0.0.1"]
 
@@ -192,14 +197,14 @@ def _load_ips_gethostbyname():
         # try hostname.local, in case hostname has been short-circuited to loopback
         if not hostname.endswith(".local") and all(ip.startswith("127") for ip in PUBLIC_IPS):
             PUBLIC_IPS[:] = socket.gethostbyname_ex(socket.gethostname() + ".local")[2]
-    except socket.error:
+    except OSError:
         pass
     finally:
         PUBLIC_IPS[:] = _uniq_stable(PUBLIC_IPS)
         LOCAL_IPS.extend(PUBLIC_IPS)
 
     # include all-interface aliases: 0.0.0.0 and ''
-    LOCAL_IPS.extend(["0.0.0.0", ""])
+    LOCAL_IPS.extend(["0.0.0.0", ""])  # noqa
 
     LOCAL_IPS[:] = _uniq_stable(LOCAL_IPS)
 
@@ -210,7 +215,7 @@ def _load_ips_dumb():
     """Fallback in case of unexpected failure"""
     global LOCALHOST
     LOCALHOST = "127.0.0.1"
-    LOCAL_IPS[:] = [LOCALHOST, "0.0.0.0", ""]
+    LOCAL_IPS[:] = [LOCALHOST, "0.0.0.0", ""]  # noqa
     PUBLIC_IPS[:] = []
 
 
@@ -256,7 +261,7 @@ def _load_ips(suppress_exceptions=True):
         if not suppress_exceptions:
             raise
         # unexpected error shouldn't crash, load dumb default values instead.
-        warn("Unexpected error discovering local network interfaces: %s" % e)
+        warn("Unexpected error discovering local network interfaces: %s" % e, stacklevel=2)
     _load_ips_dumb()
 
 
