@@ -10,6 +10,8 @@ Sessions.
 """
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
+from __future__ import annotations
+
 import hashlib
 import hmac
 import json
@@ -25,8 +27,6 @@ from datetime import datetime, timezone
 from hmac import compare_digest
 
 # We are using compare_digest to limit the surface of timing attacks
-from typing import Optional, Union
-
 import zmq.asyncio
 from tornado.ioloop import IOLoop
 from traitlets import (
@@ -188,17 +188,16 @@ def default_secure(cfg: t.Any) -> None:  # pragma: no cover
     If Session.key/keyfile have not been set, set Session.key to
     a new random UUID.
     """
-    warnings.warn("default_secure is deprecated", DeprecationWarning)
-    if "Session" in cfg:
-        if "key" in cfg.Session or "keyfile" in cfg.Session:
-            return
+    warnings.warn("default_secure is deprecated", DeprecationWarning, stacklevel=2)
+    if "Session" in cfg and ("key" in cfg.Session or "keyfile" in cfg.Session):
+        return
     # key/keyfile not specified, generate new UUID:
     cfg.Session.key = new_id_bytes()
 
 
 def utcnow() -> datetime:
     """Return timezone-aware UTC timestamp"""
-    return datetime.utcnow().replace(tzinfo=utc)
+    return datetime.now(utc)
 
 
 # -----------------------------------------------------------------------------
@@ -213,7 +212,7 @@ class SessionFactory(LoggingConfigurable):
 
     logname = Unicode("")
 
-    @observe("logname")  # type:ignore[misc]
+    @observe("logname")
     def _logname_changed(self, change: t.Any) -> None:
         self.log = logging.getLogger(change["new"])
 
@@ -245,12 +244,12 @@ class Message:
     A Message can be created from a dict and a dict from a Message instance
     simply by calling dict(msg_obj)."""
 
-    def __init__(self, msg_dict: t.Dict[str, t.Any]) -> None:
+    def __init__(self, msg_dict: dict[str, t.Any]) -> None:
         """Initialize a message."""
         dct = self.__dict__
         for k, v in dict(msg_dict).items():
             if isinstance(v, dict):
-                v = Message(v)
+                v = Message(v)  # noqa
             dct[k] = v
 
     # Having this iterator lets dict(msg_obj) work out of the box.
@@ -270,14 +269,16 @@ class Message:
         return self.__dict__[k]
 
 
-def msg_header(msg_id: str, msg_type: str, username: str, session: "Session") -> t.Dict[str, t.Any]:
+def msg_header(
+    msg_id: str, msg_type: str, username: str, session: Session | str
+) -> dict[str, t.Any]:
     """Create a new message header"""
     date = utcnow()
     version = protocol_version
     return locals()
 
 
-def extract_header(msg_or_header: t.Dict[str, t.Any]) -> t.Dict[str, t.Any]:
+def extract_header(msg_or_header: dict[str, t.Any]) -> dict[str, t.Any]:
     """Given a message or header, return the header."""
     if not msg_or_header:
         return {}
@@ -568,7 +569,7 @@ class Session(Configurable):
         self._check_packers()
         self.none = self.pack({})
         # ensure self._session_default() if necessary, so bsession is defined:
-        self.session
+        self.session  # noqa
         self.pid = os.getpid()
         self._new_auth()
         if not self.key:
@@ -576,7 +577,7 @@ class Session(Configurable):
                 "Message signing is disabled.  This is insecure and not recommended!"
             )
 
-    def clone(self) -> "Session":
+    def clone(self) -> Session:
         """Create a copy of this Session
 
         Useful when connecting multiple times to a given kernel.
@@ -612,9 +613,8 @@ class Session(Configurable):
         try:
             packed = pack(msg_list)
         except Exception as e:
-            raise ValueError(
-                f"packer '{self.packer}' could not serialize a simple message: {e}"
-            ) from e
+            msg = f"packer '{self.packer}' could not serialize a simple message: {e}"
+            raise ValueError(msg) from e
 
         # ensure packed message is bytes
         if not isinstance(packed, bytes):
@@ -625,33 +625,35 @@ class Session(Configurable):
             unpacked = unpack(packed)
             assert unpacked == msg_list
         except Exception as e:
-            raise ValueError(
+            msg = (
                 f"unpacker '{self.unpacker}' could not handle output from packer"
                 f" '{self.packer}': {e}"
-            ) from e
+            )
+            raise ValueError(msg) from e
 
         # check datetime support
         msg_datetime = {"t": utcnow()}
         try:
             unpacked = unpack(pack(msg_datetime))
             if isinstance(unpacked["t"], datetime):
-                raise ValueError("Shouldn't deserialize to datetime")
+                msg = "Shouldn't deserialize to datetime"
+                raise ValueError(msg)
         except Exception:
             self.pack = lambda o: pack(squash_dates(o))
             self.unpack = lambda s: unpack(s)
 
-    def msg_header(self, msg_type: str) -> t.Dict[str, t.Any]:
+    def msg_header(self, msg_type: str) -> dict[str, t.Any]:
         """Create a header for a message type."""
         return msg_header(self.msg_id, msg_type, self.username, self.session)
 
     def msg(
         self,
         msg_type: str,
-        content: t.Optional[t.Dict] = None,
-        parent: t.Optional[t.Dict[str, t.Any]] = None,
-        header: t.Optional[t.Dict[str, t.Any]] = None,
-        metadata: t.Optional[t.Dict[str, t.Any]] = None,
-    ) -> t.Dict[str, t.Any]:
+        content: dict | None = None,
+        parent: dict[str, t.Any] | None = None,
+        header: dict[str, t.Any] | None = None,
+        metadata: dict[str, t.Any] | None = None,
+    ) -> dict[str, t.Any]:
         """Return the nested message dict.
 
         This format is different from what is sent over the wire. The
@@ -670,7 +672,7 @@ class Session(Configurable):
             msg["metadata"].update(metadata)
         return msg
 
-    def sign(self, msg_list: t.List) -> bytes:
+    def sign(self, msg_list: list) -> bytes:
         """Sign a message with HMAC digest. If no auth, return b''.
 
         Parameters
@@ -687,9 +689,9 @@ class Session(Configurable):
 
     def serialize(
         self,
-        msg: t.Dict[str, t.Any],
-        ident: t.Optional[t.Union[t.List[bytes], bytes]] = None,
-    ) -> t.List[bytes]:
+        msg: dict[str, t.Any],
+        ident: list[bytes] | bytes | None = None,
+    ) -> list[bytes]:
         """Serialize the message components to bytes.
 
         This is roughly the inverse of deserialize. The serialize/deserialize
@@ -751,16 +753,16 @@ class Session(Configurable):
 
     def send(
         self,
-        stream: Optional[Union[zmq.sugar.socket.Socket, ZMQStream]],
-        msg_or_type: t.Union[t.Dict[str, t.Any], str],
-        content: t.Optional[t.Dict[str, t.Any]] = None,
-        parent: t.Optional[t.Dict[str, t.Any]] = None,
-        ident: t.Optional[t.Union[bytes, t.List[bytes]]] = None,
-        buffers: t.Optional[t.List[bytes]] = None,
+        stream: zmq.sugar.socket.Socket | ZMQStream | None,
+        msg_or_type: dict[str, t.Any] | str,
+        content: dict[str, t.Any] | None = None,
+        parent: dict[str, t.Any] | None = None,
+        ident: bytes | list[bytes] | None = None,
+        buffers: list[bytes] | None = None,
         track: bool = False,
-        header: t.Optional[t.Dict[str, t.Any]] = None,
-        metadata: t.Optional[t.Dict[str, t.Any]] = None,
-    ) -> t.Optional[t.Dict[str, t.Any]]:
+        header: dict[str, t.Any] | None = None,
+        metadata: dict[str, t.Any] | None = None,
+    ) -> dict[str, t.Any] | None:
         """Build and send a message via stream or socket.
 
         The message format used by this function internally is as follows:
@@ -809,7 +811,7 @@ class Session(Configurable):
             track = False
 
         if isinstance(stream, zmq.asyncio.Socket):
-            assert stream is not None
+            assert stream is not None  # type:ignore[unreachable]
             stream = zmq.Socket.shadow(stream.underlying)
 
         if isinstance(msg_or_type, (Message, dict)):
@@ -825,7 +827,7 @@ class Session(Configurable):
                 header=header,
                 metadata=metadata,
             )
-        if self.check_pid and not os.getpid() == self.pid:
+        if self.check_pid and os.getpid() != self.pid:
             get_logger().warning("WARNING: attempted to send message from fork\n%s", msg)
             return None
         buffers = [] if buffers is None else buffers
@@ -835,9 +837,10 @@ class Session(Configurable):
             else:
                 try:
                     # check to see if buf supports the buffer protocol.
-                    view = memoryview(buf)  # type:ignore[assignment]
+                    view = memoryview(buf)
                 except TypeError as e:
-                    raise TypeError("Buffer objects must support the buffer protocol.") from e
+                    emsg = "Buffer objects must support the buffer protocol."
+                    raise TypeError(emsg) from e
             # memoryview.contiguous is new in 3.3,
             # just skip the check on Python 2
             if hasattr(view, "contiguous") and not view.contiguous:
@@ -871,10 +874,10 @@ class Session(Configurable):
     def send_raw(
         self,
         stream: zmq.sugar.socket.Socket,
-        msg_list: t.List,
+        msg_list: list,
         flags: int = 0,
         copy: bool = True,
-        ident: t.Optional[t.Union[bytes, t.List[bytes]]] = None,
+        ident: bytes | list[bytes] | None = None,
     ) -> None:
         """Send a raw message via ident path.
 
@@ -911,7 +914,7 @@ class Session(Configurable):
         mode: int = zmq.NOBLOCK,
         content: bool = True,
         copy: bool = True,
-    ) -> t.Tuple[t.Optional[t.List[bytes]], t.Optional[t.Dict[str, t.Any]]]:
+    ) -> tuple[list[bytes] | None, dict[str, t.Any] | None]:
         """Receive and unpack a message.
 
         Parameters
@@ -925,8 +928,8 @@ class Session(Configurable):
             [idents] is a list of idents and msg is a nested message dict of
             same format as self.msg returns.
         """
-        if isinstance(socket, ZMQStream):
-            socket = socket.socket
+        if isinstance(socket, ZMQStream):  # type:ignore[unreachable]
+            socket = socket.socket  # type:ignore[unreachable]
         if isinstance(socket, zmq.asyncio.Socket):
             socket = zmq.Socket.shadow(socket.underlying)
 
@@ -949,8 +952,8 @@ class Session(Configurable):
             raise e
 
     def feed_identities(
-        self, msg_list: t.Union[t.List[bytes], t.List[zmq.Message]], copy: bool = True
-    ) -> t.Tuple[t.List[bytes], t.Union[t.List[bytes], t.List[zmq.Message]]]:
+        self, msg_list: list[bytes] | list[zmq.Message], copy: bool = True
+    ) -> tuple[list[bytes], list[bytes] | list[zmq.Message]]:
         """Split the identities from the rest of the message.
 
         Feed until DELIM is reached, then return the prefix as idents and
@@ -985,7 +988,8 @@ class Session(Configurable):
                     failed = False
                     break
             if failed:
-                raise ValueError("DELIM not in msg_list")
+                msg = "DELIM not in msg_list"
+                raise ValueError(msg)
             idents, msg_list = msg_list[:idx], msg_list[idx + 1 :]
             return [bytes(m.bytes) for m in idents], msg_list
 
@@ -1015,10 +1019,10 @@ class Session(Configurable):
 
     def deserialize(
         self,
-        msg_list: t.Union[t.List[bytes], t.List[zmq.Message]],
+        msg_list: list[bytes] | list[zmq.Message],
         content: bool = True,
         copy: bool = True,
-    ) -> t.Dict[str, t.Any]:
+    ) -> dict[str, t.Any]:
         """Unserialize a msg_list to a nested message dict.
 
         This is roughly the inverse of serialize. The serialize/deserialize
@@ -1055,7 +1059,8 @@ class Session(Configurable):
         if self.auth is not None:
             signature = msg_list[0]
             if not signature:
-                raise ValueError("Unsigned Message")
+                msg = "Unsigned Message"
+                raise ValueError(msg)
             if signature in self.digest_history:
                 raise ValueError("Duplicate Signature: %r" % signature)
             if content:
@@ -1063,9 +1068,11 @@ class Session(Configurable):
                 self._add_digest(signature)
             check = self.sign(msg_list[1:5])
             if not compare_digest(signature, check):
-                raise ValueError("Invalid Signature: %r" % signature)
+                msg = "Invalid Signature: %r" % signature
+                raise ValueError(msg)
         if not len(msg_list) >= minlen:
-            raise TypeError("malformed message, must have at least %i elements" % minlen)
+            msg = "malformed message, must have at least %i elements" % minlen
+            raise TypeError(msg)
         header = self.unpack(msg_list[1])
         message["header"] = extract_dates(header)
         message["msg_id"] = header["msg_id"]
@@ -1087,11 +1094,12 @@ class Session(Configurable):
         # adapt to the current version
         return adapt(message)
 
-    def unserialize(self, *args: t.Any, **kwargs: t.Any) -> t.Dict[str, t.Any]:
+    def unserialize(self, *args: t.Any, **kwargs: t.Any) -> dict[str, t.Any]:
         """**DEPRECATED** Use deserialize instead."""
         # pragma: no cover
         warnings.warn(
             "Session.unserialize is deprecated. Use Session.deserialize.",
             DeprecationWarning,
+            stacklevel=2,
         )
         return self.deserialize(*args, **kwargs)
