@@ -34,7 +34,7 @@ numbers.Real.register(MyFloat)
 
 def test_parse_date_invalid():
     assert jsonutil.parse_date(None) is None
-    assert jsonutil.parse_date("") == ""  # noqa
+    assert jsonutil.parse_date("") == ""
     assert jsonutil.parse_date("invalid-date") == "invalid-date"
 
 
